@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "../config.h"
+#include <config.h>
 #include <sys/types.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -278,47 +278,20 @@ static const struct _vte_iso2022_map _vte_iso2022_map_wide_G[] = {
 #include "unitable.CNS11643"
 };
 
-#include "uniwidths"
-
 static gint
 _vte_direct_compare(gconstpointer a, gconstpointer b)
 {
 	return GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b);
 }
 
-static gboolean
-_vte_iso2022_is_ambiguous_ht(gunichar c)
-{
-	static GHashTable *ambiguous;
-	if (G_UNLIKELY (ambiguous == NULL)) {
-		gpointer p;
-		gsize i;
-		ambiguous = g_hash_table_new (NULL, NULL);
-		for (i = 0; i < G_N_ELEMENTS(_vte_iso2022_ambiguous_chars); i++) {
-			p = GINT_TO_POINTER(_vte_iso2022_ambiguous_chars[i]);
-			g_hash_table_insert(ambiguous, p, p);
-		}
-	}
-
-	return g_hash_table_lookup(ambiguous, GINT_TO_POINTER(c)) != NULL;
-}
 static inline gboolean
 _vte_iso2022_is_ambiguous(gunichar c)
 {
-	gsize i;
-	for (i = 0; i < G_N_ELEMENTS(_vte_iso2022_unambiguous_ranges); i++) {
-		if ((c >= _vte_iso2022_unambiguous_ranges[i].start) &&
-		    (c <= _vte_iso2022_unambiguous_ranges[i].end)) {
-			return FALSE;
-		}
-	}
-	for (i = 0; i < G_N_ELEMENTS(_vte_iso2022_ambiguous_ranges); i++) {
-		if ((c >= _vte_iso2022_ambiguous_ranges[i].start) &&
-		    (c <= _vte_iso2022_ambiguous_ranges[i].end)) {
-			return TRUE;
-		}
-	}
-	return _vte_iso2022_is_ambiguous_ht (c);
+	/* ASCII chars are not ambiguous */
+	if (G_LIKELY (c < 0x80))
+		return FALSE;
+
+	return g_unichar_iswide (c) != g_unichar_iswide_cjk (c);
 }
 
 /* If we only have a codepoint, guess what the ambiguous width should be based
@@ -370,6 +343,7 @@ _vte_iso2022_ambiguous_width(struct _vte_iso2022_state *state)
 		"gb18030",
 		"gb2312",
 		"gbk",
+		"shiftjis",
 		"tcvn",
 	};
 	gsize i, j;
@@ -405,9 +379,9 @@ _vte_iso2022_ambiguous_width(struct _vte_iso2022_state *state)
 	 */
 	if (strcmp (codeset, "utf8") == 0 && g_getenv("VTE_CJK_WIDTH") != NULL) {
 	  const char *env = g_getenv ("VTE_CJK_WIDTH");
-	  if (g_ascii_strcasecmp (env, "narrow"))
+	  if ((g_ascii_strcasecmp (env, "narrow")==0) || (g_ascii_strcasecmp (env, "0")==0))
 	    return 1;
-	  if (g_ascii_strcasecmp (env, "wide"))
+	  if ((g_ascii_strcasecmp (env, "wide")==0) || (g_ascii_strcasecmp (env, "1")==0))
 	    return 2;
 	  else
 	    return _vte_iso2022_ambiguous_width_guess ();
@@ -776,9 +750,9 @@ _vte_iso2022_state_new(const char *native_codeset,
 	state->current = 0;
 	state->override = -1;
 	state->g[0] = 'B';
-	state->g[1] = '0';
-	state->g[2] = 'J';
-	state->g[3] = WIDE_FUDGE + 'D';
+	state->g[1] = 'B';
+	state->g[2] = 'B';
+	state->g[3] = 'B';
 	state->codeset = native_codeset;
 	state->native_codeset = state->codeset;
 	if (native_codeset == NULL) {
@@ -1246,7 +1220,9 @@ process_cdata(struct _vte_iso2022_state *state, const guchar *cdata, gsize lengt
 		} while ((inbytes > 0) && !stop);
 
 		/* encode any ambiguous widths and skip blanks */
-		for (i = j = 0; buf + i < outbuf; i++) {
+		j = gunichars->len;
+		g_array_set_size(gunichars, gunichars->len + outbuf-buf);
+		for (i = 0; buf + i < outbuf; i++) {
 			c = buf[i];
 			if (G_UNLIKELY (c == '\0')) {
 				/* Skip the padding character. */
@@ -1256,10 +1232,9 @@ process_cdata(struct _vte_iso2022_state *state, const guchar *cdata, gsize lengt
 				width = ambiguous_width;
 				c = _vte_iso2022_set_encoded_width(c, width);
 			}
-			buf[j++] = c;
+			g_array_index(gunichars, gunichar, j++) = c;
 		}
-		/* And append the unichars to the GArray. */
-		g_array_append_vals(gunichars, buf, j);
+		gunichars->len = j;
 
 		/* Done. */
 		processed = length - inbytes;
@@ -1269,6 +1244,8 @@ process_cdata(struct _vte_iso2022_state *state, const guchar *cdata, gsize lengt
 				     &or_mask, &and_mask);
 		i = 0;
 		acc = 0;
+		j = gunichars->len;
+		g_array_set_size(gunichars, gunichars->len + length);
 		do {
 			if (i < length) {
 				acc = (acc << 8) | cdata[i];
@@ -1301,7 +1278,7 @@ process_cdata(struct _vte_iso2022_state *state, const guchar *cdata, gsize lengt
 							"%04x\n", acc, c);
 					c = _vte_iso2022_set_encoded_width(c, width);
 				}
-				g_array_append_val(gunichars, c);
+				g_array_index(gunichars, gunichar, j++) = c;
 				if (single) {
 					break;
 				}
@@ -1309,6 +1286,7 @@ process_cdata(struct _vte_iso2022_state *state, const guchar *cdata, gsize lengt
 			}
 		} while (i < length);
 		processed = i;
+		gunichars->len = j;
 	}
 	return processed;
 }
@@ -1825,7 +1803,7 @@ main(int argc, char **argv)
 			}
 		}
 		_vte_buffer_append(buffer, string->str, string->len);
-		_vte_iso2022_process(state, buffer, gunichars);
+		_vte_iso2022_process(state, buffer, _vte_buffer_length (buffer), gunichars);
 		g_string_free(string, TRUE);
 	} else {
 		for (i = 0; i < G_N_ELEMENTS(strings); i++) {
@@ -1833,7 +1811,7 @@ main(int argc, char **argv)
 			_vte_buffer_append(buffer, string->str, string->len);
 			g_string_free(string, TRUE);
 			if (strings[i].process) {
-				_vte_iso2022_process(state, buffer, gunichars);
+				_vte_iso2022_process(state, buffer, _vte_buffer_length (buffer), gunichars);
 			}
 		}
 	}
